/**
 * @file registers the simpleBox toolbar button and binds functionality to it.
 */

import {Plugin} from 'ckeditor5/src/core';
import {ButtonView, clickOutsideHandler, ContextualBalloon} from 'ckeditor5/src/ui';
import DivManagerView from './divmanagerview';
import getRangeText from "./utils";

export default class DivManagerUI extends Plugin {
  static get requires() {
    return [ContextualBalloon];
  }

  init() {
    const editor = this.editor;

    // Create the balloon and the form view.
    this._balloon = this.editor.plugins.get(ContextualBalloon);
    this.formView = this._createFormView();

    // Register the button in the editor's UI component factory.
    editor.ui.componentFactory.add('divManager', () => {
      const button = new ButtonView();

      button.set({
        label: 'DIV',
        tooltip: true,
        withText: true
      });

      this.listenTo(button, 'execute', () => {
        this._showUI();
      });

      return button;
    });
  }

  _createFormView() {
    const editor = this.editor;
    const formView = new DivManagerView(editor.locale);

    // Execute the command after clicking the "Save" button.
    this.listenTo(formView, 'submit', () => {
      const value = {
        content:   formView.textInputView.fieldView.element.value,
        title:     formView.titleInputView.fieldView.element.value,
        id:        formView.idInputView.fieldView.element.value,
        langcode:  formView.langcodeInputView.fieldView.element.value,
        className: formView.classInputView.fieldView.element.value,
        rawStyles: formView.styleInputView.fieldView.element.value,
      };
      editor.execute('addDiv', value);

      // Hide the form view after submit.
      this._hideUI();
    });

    // Hide the form view after clicking the "Cancel" button.
    this.listenTo(formView, 'cancel', () => {
      this._hideUI();
    });

    // Hide the form view when clicking outside the balloon.
    clickOutsideHandler({
      emitter: formView,
      activator: () => this._balloon.visibleView === formView,
      contextElements: [this._balloon.view.element],
      callback: () => this._hideUI()
    });

    return formView;
  }

  _getBalloonPositionData() {
    const view = this.editor.editing.view;
    const viewDocument = view.document;
    let target = null;

    // Set a target position by converting view selection range to DOM.
    target = () => view.domConverter.viewRangeToDom(viewDocument.selection.getFirstRange());

    return {
      target
    };
  }

  _showUI() {
    const selection = this.editor.model.document.selection;

    this._balloon.add({
      view: this.formView,
      position: this._getBalloonPositionData()
    });

    // Disable the input when the selection is not collapsed.
    this.formView.textInputView.isEnabled = selection.getFirstRange().isCollapsed;

    this.formView.textInputView.fieldView.value = getRangeText(selection.getFirstRange());
    this.formView.titleInputView.fieldView.value = '';

    this.formView.focus();
  }

  _hideUI() {
    // Clear the input field values and reset the form.
    this.formView.textInputView.fieldView.value = '';
    this.formView.idInputView.fieldView.value = '';
    this.formView.classInputView.fieldView.value = '';
    this.formView.langcodeInputView.fieldView.value = '';
    this.formView.styleInputView.fieldView.value = '';
    this.formView.titleInputView.fieldView.value = '';
    this.formView.element.reset();

    this._balloon.remove(this.formView);

    // Focus the editing view after inserting the div so the user can start typing the content
    // right away and keep the editor focused.
    this.editor.editing.view.focus();
  }
}
